//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
// quikGL.cpp: implementation of the CquikGL class.
//
//	SEE quikGL.h for explanation, instructions.
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "quikGL.h"
#include <math.h>						// for trig functions (sin(), etc.)

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
#define BEEP whatsup()				// diagnostic debug action


void CquikGL::whatsup(void)
//------------------------------------------------------------------------------
{// BREAKPOINT HERE to diagnose warnings through the call stack.
	MessageBeep(MB_ICONASTERISK);
}

CquikGL::CquikGL()
//------------------------------------------------------------------------------
// Default constructor.
{
	m_hRC = NULL;					// invalid rendering context handle.
	m_bAnimate = TRUE;				// initially, enable animation
}

CquikGL::~CquikGL()
//------------------------------------------------------------------------------
// Default destructor, if any.
{
}

void CquikGL::PreCreate_GL(CREATESTRUCT& cs)
//------------------------------------------------------------------------------
// Call this from the "PreCreateWindow()" function made for you by ClassWizard
// for the CProj1View class.  This happens just BEFORE Windows makes an on-
// screen window, to allow us it's style and other parameters.
{
	// Set child window(s) to clip each other; the window on top
	// obscures the window underneath it.
	// REQUIRED!!--otherwise 'SetPixelFormat() returns error!
	cs.style |= WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
}

void CquikGL::OnSize_GL(CClientDC& clientDC, int cx, int cy)
//------------------------------------------------------------------------------
// Change the viewing geometry to fit a window of cx,cy pixels.
{
	m_isOK = wglMakeCurrent(clientDC.m_hDC, m_hRC);		// Use OpenGL rendering
												// context made in onCreateGL()
	if(m_isOK==FALSE) error_GL("OnSize_GL::makeCurrent");
	// Tie the OpenGL rendering context handle m_hRC (made in PreCreate_GL()) 
	// to the MSWindows DeviceContext (DC) for the Client part of window
	// (untie it on exit from this fcn). 
	// Why? so Windows won't ignore the OpenGL commands in this fcn!

    glMatrixMode(GL_PROJECTION);		// Clear any previous viewing transform
    glLoadIdentity();					// matrix; replace it with orthographic
										// projection (4x4 identity matrix).
	//Now set the world->eye space projection matrix.
	// THE HARD WAY: directly specify the viewing volume........................
					//!!!CAREFUL! Default OpenGL camera position is at origin, 
					// aimed in *** -Z *** direction!!! near=1, far=2 puts the
					// near,far clipping planes at z=-1, z=-2 respectively!!!!
//  glFrustum(-1.0,1.0,-1.0,1.0, 1.0, 4.0);	// left,rt,top,bott, near,far.
//	glTranslatef(0.0f,0.0f, -1.4f);			// Translate CAMERA backwards
//	if(cx<cy)								// Keep square viewport, but tie						
//		glViewport(0,0,cx,cx);				// it to lower-left window corner				
//	else									// and use smaller window dim.
//		glViewport(0,0,cy,cy);					// (left, bottom, width, height)
	//..........................................................................
	// THE EASY WAY: use the 'perspective' matrix that sets 'field-of-view', 
#define JT_FOV			30.0f				// horizontal field-of-view(degrees)
#define JT_NEAR_CLIP	 1.0f				// 3D 'near' clipping plane depth
#define JT_FAR_CLIP		 5.0f				// 3D 'far' clipping plane depth.

GLdouble gldAspect;							// display window aspect ratio.
											// (assume pixels are square)
GLfloat viewCtr;							// where to find MODELVIEW coord's
											// origin along the camera's z-axis.
    gldAspect = (GLdouble) cx/ (GLdouble) cy;	// find window aspect ratio,
	viewCtr = 0.5f*(JT_NEAR_CLIP+JT_FAR_CLIP);// and middle of viewport.
    gluPerspective(	JT_FOV, 				// Set field-of-view,
					gldAspect,				// (width/height) ratio. Match this
											// window's #_of_x_pix/#_of_ypixels
					JT_NEAR_CLIP,			// near, far clip planes.
					JT_FAR_CLIP);
	glTranslatef(0.0f,0.0f,-viewCtr);		// put world origin at object ctr.
	glViewport(0,0,cx,cy);					// Match OpenGL pixels to screen.
	//..........................................................................
	wglMakeCurrent(clientDC.m_hDC, NULL);	// Swap out old rendering context.
											// We're done!
}

void CquikGL::OnCreate_GL(CClientDC& clientDC)
//------------------------------------------------------------------------------
// Call this from inside "OnCreate(LPCREATESTRUCT lpCreateStruct)" that is part
// of the CProj1View class.  This function sets up the display plane used for
// openGL drawing: bit depths, etc., creates a rendering context, and makes an
// example image for texture mapping.
{
int nPixelFormat;
BOOL bResult;
PIXELFORMATDESCRIPTOR pfd;				// OpenGL's struct for screen pixel;

    // Set the **REQUESTED** OpenGL pixel format:.........................
	memset(&pfd, 0, sizeof(PIXELFORMATDESCRIPTOR));		// clear everything,
	pfd.nSize      = sizeof(PIXELFORMATDESCRIPTOR);		// record struct size,
	pfd.nVersion   = 1;						// Version #.					
	pfd.dwFlags =  PFD_DOUBLEBUFFER  |		// Use double buffer;(OpenGL only)
				//	PFD_SUPPORT_GDI   |		// Use single-buffer; lets BOTH 
											// OpenGL and GDI calls write to 
											// window (using PFD_DOUBLEBUFFER 
											// prevents GDI calls in window).
                    PFD_SUPPORT_OPENGL |	// Duh!.
                    PFD_DRAW_TO_WINDOW;		// draw to window, not a bitmap
											// in memory or device surface,
	pfd.iPixelType = PFD_TYPE_RGBA;			//	32-bit RGBA OPenGL pixels
	pfd.cColorBits = 24;
	pfd.cDepthBits = 32;					// 32-bit depth buffer
	pfd.iLayerType = PFD_MAIN_PLANE;		// (the only supported way)

	nPixelFormat = ChoosePixelFormat(clientDC.m_hDC, &pfd);
											// Get best match to available hdwe:
											// returns index to actual PFD.
	if (nPixelFormat == 0)					// ?NO match at all? 
	{
		error_GL("CquikGL::OnCreate_GL()");	// report error details.
		BEEP;
	}
	TRACE("Pixel Format %d\r\n",nPixelFormat);	// show me what you'll do,
	bResult = SetPixelFormat(clientDC.m_hDC, nPixelFormat, &pfd); // set it.
	if (bResult==FALSE)
	{
		TRACE("SetPixelFormat Failed %d\r\n",GetLastError());
		BEEP;
	}
	if(pfd.dwFlags & PFD_NEED_PALETTE)		// does hardware require palette?
	{										// TOO BAD! I won't support it.
		TRACE("Display Hardware Requires Palette! I won't do that!\r\n");
		BEEP;
	//	CreateRGBPalette(dc.m_hDC) ;			// make the palette?
	}
	// Ready. Make our OpenGL Rendering Context; keep its handle for rendering.
	m_hRC = wglCreateContext(clientDC.m_hDC);
	if (!m_hRC)
	{
		error_GL("CquikGL::OnCreate_GL()");		// make labeled error message,
		BEEP;									// complain.
	}
	wglMakeCurrent(clientDC.m_hDC, m_hRC);		// Start up the new OpenGL
												// rendering context, do some
												// one-time init. tasks:
	//..........................RENDER STATE............................
	glEnable(GL_DEPTH_TEST);					// Enable z-buffer depth tests
	// DON'T enable backface culling if you want to see backsides of polys.
	//	glEnable(GL_CULL_FACE);					// Enable backface culling
	//	glCullFace(GL_FRONT);					// Choose FRONT or BACK face
												// (default is backface cull)
	//...................CREATE A TEST-PATTERN TEXTURE MAP......................
	m_vimg.makeTestImgMesh();
	//.........................LOAD A FONT FOR TEXT.............................
	// Windows' OpenGL implementation ruins all the Windows GDI drawing calls, 
	// so they include an awkward hack to allow text display: Create a set of 
	// 256 display lists in OpenGL, one for every  symbol or 'glyph' of the 
	// font we'll use. 
											// load a Windows font into DC:
	SelectObject(clientDC.m_hDC, GetStockObject (SYSTEM_FONT));  	
	m_fontID = glGenLists(256);				// make 256 contiguous display lists
											// all empty, return ID# for 1st one
	m_isOK = wglUseFontBitmaps (clientDC.m_hDC, 0, 255, m_fontID); 
	if(m_isOK==FALSE) error_GL("OnCreateGL::FontSetup"); // report any errs
	SelectObject(clientDC.m_hDC, NULL);		// Swap out the font we used.

// Set up some lighting........................................................
// OpenGL transforms EVERYTHING by GL_MODELVIEW matrix, then GL_PROJECTION 
// matrix, then viewport matrix to stuff onscreen.  We want lighting to stay
// fixed to the camera's viewpoint, so we temporarily nullify the GL_MODELVIEW
// matrix by pushing its current value onto the matrix stack, replacing it with
// the identity matrix.  After all lighting settings are done, then pop the old
// MODELVIEW matrix off the stack before we draw anything.
	glMatrixMode(GL_MODELVIEW);					// Pick modelview matrix and
	glPushMatrix();								// push it onto its own stack.
	glLoadIdentity();							// Benign matrix--does nothing.
	error_GL("OnCreateGL::Light Coords");		// report any errors.
	
GLfloat lite0pos[4]= {0.5f, 0.5f, 0.5f, 1.0f};	// Light 0 position
GLfloat lite0amb[4]= {0.6f, 0.6f, 0.6f, 1.0f};	// Light 0 ambient illum.
GLfloat lite0dif[4]= {0.9f, 0.9f, 0.9f, 1.0f};	// Light 0 diffuse illum.
GLfloat lite0spec[4]={1.0f, 1.0f, 1.0f, 1.0f};	// Light 0 specular illum.

	//(up to 8 independent light sources allowed in OpenGL: GL_LIGHT0 - 7)
	glLightfv(GL_LIGHT0, GL_POSITION, lite0pos);	// set light 0 values;
	glLightfv(GL_LIGHT0, GL_AMBIENT, lite0amb);	
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lite0dif);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lite0spec);
	glEnable(GL_LIGHT0);						// turn on Light 0.
	// DONE with all lighting settings (materials settings are next). Restore
	glPopMatrix();								// the orig. MODELVIEW matrix
	error_GL("OnCreateGL::lights done");		// before drawing the model.

	// Set up a surface material...............................................
GLfloat mat_ambient[4] = {0.5f, 0.1f, 0.1f, 1.0f};	// set ambient glow
GLfloat mat_diffuse[4] = {1.0f, 0.2f, 0.2f, 1.0f};	// set diffuse reflectance
GLfloat mat_specular[4]= {1.0f, 1.0f, 1.0f, 1.0f};	// set specular reflectance
GLfloat mat_shininess[1] = {40.0f};					// set specular sharpness

											// for polygon FRONT surfaces only,
											// set amb, diff, spec reflectances.
	glMaterialfv(GL_FRONT, GL_AMBIENT,	mat_ambient); 
	glMaterialfv(GL_FRONT, GL_DIFFUSE,	mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS,mat_shininess);
											// for polygon BACK surfaces only,
											// set zero values for everything.
GLfloat nullFour[4] = {0.0f, 0.0f, 0.0f, 1.0f};		// 'black' values for
GLfloat nullOne[1] = {1.0f};						// back sides of polys.
	glMaterialfv(GL_BACK, GL_AMBIENT, nullFour);	// ?WHY DOESN'T THIS WORK?
	glMaterialfv(GL_BACK, GL_DIFFUSE, nullFour);	// check is glLightModelf()
	glMaterialfv(GL_BACK, GL_SPECULAR, nullFour);	// GL_LIGHT_MODEL_TWO_SIDE?
	glMaterialfv(GL_BACK, GL_SHININESS,nullOne);
	// Now define all our lighting coords (fixed w.r.t. camera)
	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE,1.0);	// enable different lighting
												// for backside of polys
	glShadeModel(GL_SMOOTH);					// Enable pixel-by-pixel shading
	glEnable(GL_LIGHTING);						// enable OpenGL lighting,

	// OpenGL bug? Must USE the lighting & materials settings before they can
	// be push/popped from the GL attribute stack, as happens if CquikAxes is
	// first item drawn. To avoid the problem, draw a transient triangle that 
	// will disappear on after 1st frame of animation:
/*	glBegin(GL_TRIANGLES);
	glNormal3f(0.0f,0.0f,1.0f); glVertex3f(0.0f,0.0f,0.0f);
	glNormal3f(0.1f,0.0f,1.0f); glVertex3f(0.0f,0.0f,0.0f);
	glNormal3f(0.0f,0.1f,1.0f); glVertex3f(0.0f,0.0f,0.0f);
	glEnd();
*/	//..............PUT YOUR OWN PROJECT'S INITIALIZATION HERE..................

	//..................................END.....................................
	wglMakeCurrent(clientDC.m_hDC, NULL);	// Swap out old rendering context.
											// We're done!
}

void CquikGL::drawOpenGL(CDC* pDC)
//------------------------------------------------------------------------------
// Called by CProj1View::OnDraw class like this: drawOpenGL(pDC); where 'pDC' 
// is pointer to Window's current Device Context, part of the GDI system that
// Windows uses for basic on-screen drawing. It is needed to hook the on-screen
// window to OpenGL drawing routines.
{
	m_isOK = wglMakeCurrent(pDC->m_hDC, m_hRC);	// Use OpenGL rendering context
												// made in PreCreate_GL(),
	if(m_isOK==FALSE) error_GL("drawOpenGL::context");	// report any errors
												// clear the color & z buffers:
    glClearColor(0.4f, 0.4f, 0.4f, 1.0f);		// Set background color	
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	

	// Draw labeled axes at origin..............................................
	m_P2axes.drawMe(m_fontID);					// Draw 3-Space axes.

	// Draw a moving, simple-colored ray........................................
	glPushAttrib(GL_LIGHTING_BIT);			// save current lighting state
	glDisable(GL_LIGHTING);					// turn off lighting for point,
	glColor3f(0.0f,1.0f,1.0f);				// set current color to cyan.
	drawP2point(m_Zmove,0.1f,-1.0f);		// draw a moving ray
	glPopAttrib();							// restore prev. lighting state.

	// Draw a waving triangle with complex lighting.............................
	glBegin(GL_TRIANGLES);						
		glVertex3f(-0.2f +m_Xmove,-0.4f,-0.1f);	// make a lit triangle from
		glVertex3f(-0.4f,-0.1f+m_Ymove,-0.3f);	// current material & lights.
		glVertex3f(-0.4f,-0.4f,-0.1f+m_Zmove);	// tickGL() makes it move.
	glEnd();

	//Write simple-colored Text................................................
		// We want to draw in world-space coords(like the lights and camera)
		// so we temporarily disable the modelview matrix:
	glMatrixMode(GL_MODELVIEW);				// Pick modelview matrix and
	glPushMatrix();							// push it onto its own stack.
	glLoadIdentity();						// Benign matrix--does nothing.
	glListBase(m_fontID);			// point to start of the font display lists
									// that we set up in OnCreate_GL().
	glPushAttrib(GL_LIGHTING_BIT);			// Save current lighting,
	glDisable(GL_LIGHTING);					// then set what we want:
	glColor3f(0.0f,1.0f,0.0f);				// set color to green.	
				// Now call a display list for each character in a string:
	m_msg = "Display a Test Message";
	glRasterPos3f(-0.5f,0.50f,0.5f);		// 3D positioning for text line
	glCallLists(strlen(m_msg), GL_UNSIGNED_BYTE, m_msg); 
	glColor3f(1.0f,1.0f,0.0f);				// set color to yellow.	
	m_msg = "in OpenGL on Windows";
	glRasterPos3f(-0.5f,0.46f,0.5f);		// 3D positioning for text line
	glCallLists(strlen(m_msg), GL_UNSIGNED_BYTE, m_msg);
	glPopAttrib();							// restore lighting data.
	glPopMatrix();							// restore modelview matrix.

	// Texture Map I............................................................
	// Use the test image created in OnCreate_GL(); copy image to screen 
	// location computed from **3D** position:
	glRasterPos3f(0.3f,-0.3f,0.0f);		// Find the 3D position, then
	switch(m_vimg.m_Tzsize)				// draw at that screen position.
	{
	case 3:								// for RGB-only image;
		glDrawPixels(m_vimg.m_Txsize,m_vimg.m_Tysize, GL_RGB, 
			GL_UNSIGNED_BYTE, m_vimg.m_pTimg);
		break;
	case 4:								// for RGBA image:
		glDrawPixels(m_vimg.m_Txsize,m_vimg.m_Tysize, GL_RGBA, 
			GL_UNSIGNED_BYTE, m_vimg.m_pTimg);
		break;
	default:
		BEEP;							// ?!?!? weird texture image size!
		break;
	}
	error_GL("drawOpenGL::screenTexmap");	// report errors, if any.

	//Texture Map II............................................................
	// Use the test image created in OnCreate_GL(), display as a texture map
	// on a 3D rectangle.
	m_vimg.drawMe();

	//..........................................................................
    glFlush();							// Trigger OpenGL to draw everything
	SwapBuffers(pDC->m_hDC) ;			//Swap the screen's double-buffers.
	wglMakeCurrent(pDC->m_hDC, NULL);	// Swap out our old rendering context.
										// We're done!
}

void CquikGL::error_GL(char *label)
//------------------------------------------------------------------------------
// If OpenGL had any errors, print string found at 'label' to report where it
// happened in our code, and then print the error report. Otherwise do nothing.
// (NOTE: the 'TRACE()' macro works just like the 'printf()' function, EXCEPT 
//    output appears at bottom of the VisualC++6.0 window on the 'debug' tab.)
{
GLenum errorno;								// OpenGL error code.

	errorno = glGetError();					
	if (errorno != GL_NO_ERROR)				// was there any unreported error?
	{										// YES: tell me about it.
		TRACE("%s had error: #(%d) %s\r\n", 
		label, errorno, gluErrorString(errorno)); // (give OpenGL's explanation)
	}
}

void CquikGL::OnDestroy_GL(void)
//------------------------------------------------------------------------------
// Call this from inside "OnDestroy()" function that is part of CProj1View 
// class.  It performs an orderly exit from OpenGL rendering for this window.
{
	m_bAnimate=FALSE;
//	wglMakeCurrent(m_hRC);				// connect to OpenGL, and
//	glDeleteLists(m_fontID,256);		// dump the display lists used for fonts 
//	wglMakeCurrent(pDC->m_hDC, NULL);	// Swap out our old rendering context.
										// We're done!
	wglMakeCurrent(NULL,NULL);			// swap out our Rendering Context,
	if (m_hRC)							// and if we haven't already,
	{
		wglDeleteContext(m_hRC);		// discard it.
		m_hRC = NULL;
	}
	wglDeleteContext(m_hRC);			// Get rid of OpenGL rendering context.
}

void CquikGL::tick_GL(void)
//------------------------------------------------------------------------------
// Advance the openGL animation by one timestep. Called by CProj1View::OnTimer
// if m_bAnimate is TRUE, and >=1 frame time has elapsed since last call.
// Actual re-drawing is triggered by  CProj1View::OnTimer() routine, by the 
// 'Invalidate(FALSE)' call.
{
static int cnt=0;
double theta;
#define IBMR_TICK_MAX	120				// cycle time for circular motion

	// Simple example of animation;
	cnt = (cnt+1)%IBMR_TICK_MAX;		// theta runs from zero to 2pi
	theta = 2.0*M_PI*(double)cnt / (double)IBMR_TICK_MAX;
	m_Xmove = 0.05f*(float)sin(theta);
	m_Ymove = 0.05f*(float)cos(theta);
	m_Zmove = 0.05f*(float)cos(theta*2.0);	
}

void CquikGL::LButtonDown(CClientDC& clientDC, CPoint pnt)
//------------------------------------------------------------------------------
// Beginning of mouse click or drag. See m_mouseNow for current mouse position
// Mouse coords count pixels; zero at upper left corner of window's client area.
{
	m_mouseStartPix = pnt;					// remember mouse start position,
	m_mouseNowPix = pnt;					// and update the 'current' value.
}

void CquikGL::LButtonUp(CClientDC& clientDC, CPoint pnt)
//------------------------------------------------------------------------------
{
	m_mouseNowPix = pnt;					// (don't change start position)
}

void CquikGL::LMouseDrag(CClientDC& clientDC, CPoint pnt)
//------------------------------------------------------------------------------
//User is dragging the mouse--see m_mouseNow and m_mousePrev for screen position
{
float angl;
GLfloat oldMat[16];						// holds previous modelview matrix

	wglMakeCurrent(clientDC.m_hDC, m_hRC);	
	// Tie the OpenGL rendering context handle m_hRC (made in PreCreate_GL()) 
	// to the MSWindows DeviceContext (DC) for the Client part of window
	// (untie it on exit from this fcn). 
	// Why? so Windows won't ignore the OpenGL commands in this fcn!

	// Everything in openGL is transformed first by the current GL_MODELVIEW 
	// matrix (model->world coords), and then by current GL_PROJECTION matrix
	// (world->screen coords), then thru viewport settings. If mouse dragging
	// modifies GL_MODELVIEW, then it would reposition objects in model coords, 
	// but we want movement in 'world' space, _after_ GL_MODELVIEW is applied. 
	// Thus we copy existing GL_MODELVIEW matrix to 'oldMat', replace it with 
	// an identity matrix, modify that with mouse movement, and then multiply 
	//	it with the matrix we saved earlier.
    glMatrixMode(GL_MODELVIEW);		// Copy the current model->world matrix.
	glGetFloatv(GL_MODELVIEW_MATRIX, oldMat); 
	glLoadIdentity();				// Replace it with the identity matrix;
									// (now we work in camera coordinates)
	// Mouse's change in position changes the model position:
		// (For on-screen translation:).................
		//	glTranslatef((float)(pnt.x - m_mouseStartPix.x)*0.002f,
		//				 -(float)(pnt.y - m_mouseStartPix.y)*0.002f, 0.0f);
										//(Windows origin is upper left)
	// --OR-- For on-screen rotation:..................
	angl =(float)((pnt.x - m_mouseStartPix.x)*(pnt.x - m_mouseStartPix.x) +
				  (pnt.y - m_mouseStartPix.y)*(pnt.y - m_mouseStartPix.y));
	angl = 0.1f*sqrtf(angl);
			// (HINT: want mouse acceleration? just drop the square root!)
	glRotatef(angl, (float)(pnt.y - m_mouseStartPix.y),
					(float)(pnt.x - m_mouseStartPix.x),0.0f);
	//.......................
	// Apply this translation(rotation) in 'camera coords'; post-multiply 
	// it with the original GL_MODELVIEW matrix we saved in oldMat
	glMultMatrixf(oldMat);
	//..........................................................................
	wglMakeCurrent(clientDC.m_hDC, NULL);	// Swap out old rendering context.
	m_mouseNowPix = pnt;					// Update current mouse position
	m_mouseStartPix = pnt;					// and the last used position.
}

void CquikGL::drawP2point(float x1, float x2, float x3)
//------------------------------------------------------------------------------
// Draw a Point in P2 space (e.g. x1/x3, x2/x3) in 3-space (x=x1,y=x2,z=x3)
// Line is L = (L1,L2,L3). L1*x1 L2
{
#define JT_EPSILON	10E-3					// min. x3 value allowed to divide.
#define JT_RAYLENGTH 2.0f					// length of rays we draw

float xx,yy,zz,norm;

	if(x3<JT_EPSILON && x3>-JT_EPSILON)		// x3 given is too small for divide
	{
		zz = 0.0f;							// so assume line is in x3=0 plane
		norm = (float)sqrt((double)(x1*x1 + x2*x2));
		xx = (JT_RAYLENGTH*x1)/norm;
		yy = (JT_RAYLENGTH*x2)/norm;
	}
	else									// usable x3
	{										// find vector length,
		norm = (float)sqrt((double)(x1*x1 + x2*x2 + x3*x3));
		xx = (JT_RAYLENGTH*x1)/norm;		// draw 
		yy = (JT_RAYLENGTH*x2)/norm;
		zz = (JT_RAYLENGTH*x3)/norm;
	}
	glBegin(GL_LINES);
	{
		glVertex3f(xx,yy,zz);				// positive endpoint
//		glVertex3f(-xx,-yy,-zz);			// negative endpoint --OR---
		glVertex3f(0.0f,0.0f,0.0f);			// origin (uncomment only one)
	}
	glEnd();
}

void CquikGL::drawP2line(float e1, float e2, float e3)
//------------------------------------------------------------------------------
// Given the P2-space line defined by E=(e1,e2,e3), draw:
//	--the corresponding line on the (x,y,0) image plane (Cartesian coords),
//		clipped to the limits of the image plane (e.g. x,y > +/- 0.5, etc.)
//		If the line is off-screen , then indicate it somehow.
//  --The corresponding half-plane in Cartesian coords (x=x1,y=x2,z=x3). Draw a
//		square, semi-transparent polygon, side-length of 2*IBMR_PLANEMAX, that
//		passes through origin and has one edge somewhere on the x,y plane.
//		Recall; every point X=(x1,x2,x3) in the plane satisfies E(dot)X = 0. 
{
#define IMBR_PLANEMAX 1.0f					// size of plane caused by P2 line.

	//Student code goes here
}

//====================
//====================
//====================
//
//		CquikAxes 
//
//====================
//====================
//====================


CquikAxes::CquikAxes()
//------------------------------------------------------------------------------
{
	labl[0] = "x1";							// set text labels for axes
	labl[1] = "x2";
	labl[2] = "x3";
}

CquikAxes::~CquikAxes()
//------------------------------------------------------------------------------
// Destructor (default)
{
}

void CquikAxes::drawMe(GLuint fontID)
//------------------------------------------------------------------------------
// Call from within CquikGL::drawOpenGL() only. (Why? fcn assumes lots of openGL
// state info has already been set by CquikGL::OnCreate_GL(), and that we have
// a valid openGL rendering context attached to the Windows device context.
// 'fontID' is the OpenGL display list number for the 1st letter in the font.
{
	// Draw the MODELVIEW coord. system axes, and rays from origin for the
	// projective 2-space (P2) where x=x1, y=x2, z=x3; draw rays for 
	// (x1=+/-0.5,x2=+/-0.5,x3=[0 to 5]).
#define AXIS_LEN	0.2f					// length of x,y,z axis markers
#define RAY_LEN		2.0f					// Length of 45 degree rays.
#define ORIGIN_SIZE 0.01f					// Size of origin blob.

	glPushAttrib(GL_LIGHTING_BIT);			// save current lighting state
	glPushAttrib(GL_COLOR_BUFFER_BIT);		// and transparency state
	glDisable(GL_LIGHTING);					// turn off lights for axes,
	glDisable(GL_BLEND);					// turn off transparency.
						// Draw axis text labels...............................
	glListBase(fontID);			// point to start of the font display lists
								// that we set up in OnCreate_GL().
	glColor3f(0.0f,0.0f,0.0f);				// black text:
	glRasterPos3f(AXIS_LEN,0.0f,0.0f);		// 3D text positioning
	glCallLists(strlen(labl[0]), GL_UNSIGNED_BYTE, labl[0]);
	glRasterPos3f(0.0f,AXIS_LEN,0.0f);		// 3D text positioning
	glCallLists(strlen(labl[1]), GL_UNSIGNED_BYTE, labl[1]); 
	glRasterPos3f(0.0f,0.0f, AXIS_LEN);		// 3D text positioning
	glCallLists(strlen(labl[2]), GL_UNSIGNED_BYTE, labl[2]); 
						// Draw axes themselves.................................
	glBegin(GL_LINES);
	glColor3f(1.0f,0.0f,0.0f);				// red x axis			
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(AXIS_LEN,0.0f,0.0f);			
	glColor3f(0.0f,1.0f,0.0f);				// green y axis			
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(0.0f,AXIS_LEN,0.0f);			
	glColor3f(0.0f,0.0f,1.0f);				// blue z axis
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(0.0f,0.0f,AXIS_LEN);	
						// Show 4 P2 rays......................................
	glColor3f(0.0f,0.0f,0.0f);				// black rays:
	glVertex3f(0.0f,0.0f,0.0f);				// 4 projection rays
	glVertex3f(-RAY_LEN,-RAY_LEN,-2.0f*RAY_LEN);
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f( RAY_LEN,-RAY_LEN,-2.0f*RAY_LEN);
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f( RAY_LEN, RAY_LEN,-2.0f*RAY_LEN);
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(-RAY_LEN, RAY_LEN,-2.0f*RAY_LEN);
	glEnd();
						// Draw an origin blob.................................
	glColor3f(1.0f,1.0f,0.0f);				// Set current color to yellow
	makeCube(ORIGIN_SIZE);

//auxSolidCube(0.02);			// DON'T USE THESE! in Windows GL implentation
//auxSolidSphere(0.02);			// they cause small memory leaks 
								// and mess up lighting/materials settings.

	glPopAttrib();						// restore original transparency state.
	glPopAttrib();						// restore original lighting state.
}
#undef AXIS_LEN					// locals only
#undef RAY_LEN
#undef ORIGIN_SIZE


void CquikAxes::makeCube(float sideLength)
//------------------------------------------------------------------------------
// Draw a cube at origin, sized by sideLength.  This is a home-made replacement
// for 'auxSolidCube()' function; the Windows implementation causes small 
// memory leaks and messes up lighting/materials settings.
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();					// Save current matrix
	glScalef(sideLength,sideLength,sideLength);	// scale to proper size
	glBegin(GL_QUADS);
	glVertex3f( 1.0f, 1.0f, 1.0f);	// +x
	glVertex3f( 1.0f,-1.0f, 1.0f);	
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);

	glVertex3f( 1.0f, 1.0f, 1.0f);	// +y
	glVertex3f( 1.0f, 1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);

	glVertex3f( 1.0f, 1.0f, 1.0f);	// +z
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);

	glVertex3f(-1.0f,-1.0f,-1.0f);	// -x
	glVertex3f(-1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);

	glVertex3f(-1.0f,-1.0f,-1.0f);	// -y
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);

	glVertex3f(-1.0f,-1.0f,-1.0f);	// -z
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glEnd();
	glPopMatrix();					// Restore original matrix.
}

void CquikGL::OnFileOpen(HWND hSrcWin)
//------------------------------------------------------------------------------
// Called when user presses an 'Open File' button or menu.  Uses Windows SDK
// common dialog box (standard) to ask user for a BMP filename. m_file2open
// holds all file string, path, filter, etc. info.
{
BOOL isOK,anim;
#define JT_MAXBUF	1024		// size of text buffers.

static char fullPath[JT_MAXBUF] = {NULL};		// full path+filename buffer.
static char fname[JT_MAXBUF] = {NULL};			// filename, extension only.
static char filt[128];
	
	anim = m_bAnimate;				// save current animation state
	m_bAnimate = FALSE;				// and STOP animating.
	// set up all the details of Windows fileOpen dialog box:
	// Make a 'file filter' string for drop-down menu:
	sprintf(filt,"%s%c%s%c%s%c%s%c","Image File *.BMP",'\0',"*.BMP",'\0',
									"All Files *.*",'\0',"*.*",'\0');
	m_file2open.hwndOwner = hSrcWin;
	m_file2open.hInstance=0;
	m_file2open.lpstrFilter = filt;
	m_file2open.lpstrCustomFilter = NULL;
	m_file2open.nMaxCustFilter = 0;
	m_file2open.nFilterIndex = 1;
	m_file2open.lpstrFile = fullPath;		// ptr to text buffer for full path
	m_file2open.lpstrFileTitle = fname;		// ptr to text buffer for filename
	m_file2open.nMaxFile = JT_MAXBUF;
	m_file2open.nMaxFileTitle = JT_MAXBUF;
	m_file2open.lpstrInitialDir = NULL;		// start in current directory
	m_file2open.lpstrTitle = "Open Image (&Mesh) Files";
	m_file2open.lpstrDefExt = "bmp";		// default file extension 
	m_file2open.Flags = OFN_FILEMUSTEXIST | OFN_LONGNAMES;
	m_file2open.lStructSize = sizeof(m_file2open);

	isOK = GetOpenFileName(&m_file2open);	// Open File dialog box...
	if(isOK==FALSE) 
	{
		m_bAnimate = anim;			// restore animation state; No file read.
		return;								
	}
	//------------------------------CALL YOUR FILE READING FCNS HERE

	m_vimg.LoadFile(&m_file2open); // Let the ImgMesh object(s) read in images
	//----------------------------------
		m_bAnimate = anim;				// Done! restore animation state.
	return;
}

void CquikGL::OnFileSave(HWND hSrcWin)
//------------------------------------------------------------------------------
// Called when user presses an 'Save File' button or menu.  Uses Windows SDK
// common dialog box (standard) to ask user for a BMP filename.
{
BOOL isOK,anim;
#define JT_MAXBUF	1024		// size of text buffers.

static char fullPath[JT_MAXBUF] = {NULL};		// full path+filename buffer.
static char fname[JT_MAXBUF] = {NULL};			// filename, extension only.
static char filt[128];
	
	anim = m_bAnimate;				// save current animation state
	m_bAnimate = FALSE;				// and STOP animating.
	// set up all the details of Windows fileOpen dialog box:
	// Make a 'file filter' string for drop-down menu:
	sprintf(filt,"%s%c%s%c%s%c%s%c","Image File *.BMP",'\0',"*.BMP",'\0',
									"All Files *.*",'\0',"*.*",'\0');
	m_file2save.hwndOwner = hSrcWin;
	m_file2save.hInstance=0;
	m_file2save.lpstrFilter = filt;
	m_file2save.lpstrCustomFilter = NULL;
	m_file2save.nMaxCustFilter = 0;
	m_file2save.nFilterIndex = 1;
	m_file2save.lpstrFile = fullPath;			// ptr to text buffer for full path
	m_file2save.lpstrFileTitle = fname;		// ptr to text buffer for filename
	m_file2save.nMaxFile = JT_MAXBUF;
	m_file2save.nMaxFileTitle = JT_MAXBUF;
	m_file2save.lpstrInitialDir = NULL;		// start in current directory
	m_file2save.lpstrTitle = "Save Image (&Mesh) Files";
	m_file2save.lpstrDefExt = "bmp";			// default file extension 
	m_file2save.Flags = OFN_LONGNAMES | OFN_OVERWRITEPROMPT;
	m_file2save.lStructSize = sizeof(m_file2save);

	isOK = GetSaveFileName(&m_file2save);	// Open File dialog box...
	if(isOK==FALSE) 
	{
		m_bAnimate = anim;			// restore animation state. No file save.
		return;								
	}
	m_bAnimate = anim;				// Done! restore animation state.
	//------------------------------CALL YOUR FILE SAVING FCNS HERE

	m_vimg.SaveFile(&m_file2open);	// Let the ImgMesh object(s) save images
	//----------------------------------
	return;
}

#undef JT_MAXBUF		//locals only!

